<?php

/**
 *
 * @wordpress-plugin
 * Plugin Name:       UBC CTLT Event Espresso 4 Modifications
 * Plugin URI:        http://ctlt.ubc.ca/
 * Description:       This plugin provides all of the additional functionality required to modify EE4 to suit our purposes
 * Version:           1.0.15
 * Author:            Richard Tape
 * Author URI:        http://blogs.ubc.ca/richardtape
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       ubc-ctlt-event-espresso-modifications
*/
// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

class UBC_CTLT_EE_Mods {

	/**
	 * $passed_template_args is an array of arguments passed to the internal EE template
	 * loader. We hook in as the template is called and then set it as a class property
	 * so we're able to use it later
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var $passed_template_args
	 */

	static $passed_template_args = false;

	/**
	 * $ticket_price is the price of a ticket. It's set so we can access it when we
	 * wouldn't normally have access in different filters
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var float $ticket_price
	 */

	static $ticket_price = false;

	/**
	 * A mapping of Question ID to user_meta keys
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (array) $question_id_user_meta_map
	 */

	static $question_id_user_meta_map = array(
		1 => 'first_name',
		2 => 'last_name',
		3 => 'user_email',
		20 => 'fac_school_college',
		21 => 'department',
		22 => 'institution',
		23 => 'attending_as',
	);


	/**
	 * The User ID of the person signed in. Used on the event-registration form
	 * as for some reason this clears the User ID
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (array) $user_id
	 */


	static $user_id = false;

	/**
	 * A way to map an error message to a message code for use when a user submits the
	 * profile field form
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (array) $ubc_ctlt_notices_map
	 */

	static $ubc_ctlt_notices_map = array(
		'email-exists' 			=> 'The email address you entered is already attached to an existing user on this system.',
		'email-required' 		=> 'We require a valid email address for you. Event registration confirmations and notifications will be sent to this address.',
		'full-name-required' 	=> 'Please tell us your full name so we are able to associate your account with your name.',
		'faculty-required' 		=> 'We require the faculty, school or college with which you are associated. Choose "other" or "N/A" as appropriate.',
		'department-required' 	=> 'We require the department with which you are associated. Choose "other" or "N/A" as appropriate.',
		'institution-required' 	=> 'We require the institution from which you will be attending. Choose "other" as appropriate.',
		'attending-as-required' => 'We need to know as what role you will be attending.',
	);


	/**
	 * The ID of the question group for our Fac/School/College, Dept., Institution and Attending As
	 * fields
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (int) $question_group_id_for_our_custom_questions
	 */
	static $question_group_id_for_our_custom_questions = 108;


	/**
	 * Domain mapping makes a mess of plugin URLs. Defining PLUGINDIR certainly helps
	 * but still breaks stuff in mu-plugins. We use this constant to set the original
	 * stuff passed to plugins_url() BEFORE domain mapping mungs it (priority 0), and
	 * then use it again (after priority 1) to de-mung it
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (array) $original_plugin_urls_before_domain_mapping
	 */

	static $original_plugin_urls_before_domain_mapping = array();

	/**
	 * Initialize our actions and filters
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function init() {

		// If a user registers from the Okanagan site, we should add them to that site and make it their default
		add_action( 'user_register', array( $this, 'user_register__okanagan_user_differences' ) );

		// Domainmapping fudges the redirect for sign in for non-admin urls for the Okanagan. Fix that.
		add_filter( 'ubc_ee_get_original_url', array( $this, 'ubc_ee_get_original_url__only_http_front_end_okanagan' ) );

		// Apparently, the most important thing ever. Howdy -> Hello
		add_filter( 'gettext', array( $this, 'gettext__howay_howdy' ), 10, 3 );

		// Allow us to have access to the template args used within this class
		add_filter( 'FHEE__EEH_Template__display_template__template_args', array( $this, 'FHEE__EEH_Template__display_template__template_args__set_class_properties_for_args' ) );
		add_filter( 'FHEE__ticket_selector_chart_template__do_ticket_inside_row', array( $this, 'FHEE__ticket_selector_chart_template__do_ticket_inside_row__set_class_properties_for_args' ), 10, 9 );

		// Adjust the ticket selector price heading
		add_filter( 'FHEE__ticket_selector_chart_template__table_header_price', array( $this, 'FHEE__ticket_selector_chart_template__table_header_price__adjust_price_header' ), 10, 2 );

		// Adjust the ticket selector Quantity heading
		add_filter( 'FHEE__ticket_selector_chart_template__table_header_qty', array( $this, 'FHEE__ticket_selector_chart_template__table_header_qty__adjust_qty' ), 10, 2 );

		// Adjust the output of the price when a ticket is free
		add_filter( 'FHEE__EEH_Template__format_currency__amount', array( $this, 'FHEE__EEH_Template__format_currency__amount__adjust_free_price_output' ), 10, 2 );

		// We do not want the 'Please note that a maximum number..' in the ticket footnote
		add_filter( 'FHEE__ticket_selector_chart_template__maximum_tickets_purchased_footnote', '__return_empty_string' );

		// EE 4.9.0 adds a filter to hide the ticket selector if there is only 1 ticket. We don't want that.
		add_filter( 'FHEE__ticket_selector_chart_template__hide_ticket_selector', '__return_false' );

		// Add user meta which enables us to have form fields filled automatically
		add_action( 'show_user_profile', array( $this, 'show_edit_user_profile__add_events_fields' ) );
		add_action( 'edit_user_profile', array( $this, 'show_edit_user_profile__add_events_fields' ) );
		add_action( 'personal_options_update', array( $this, 'personal_options_update__save_events_fields' ) );
		add_action( 'edit_user_profile_update', array( $this, 'personal_options_update_edit_user_profile_update__save_events_fields' ) );

		// Pre-fill the answer to some questions if we already have the information stored in user meta
		add_filter( 'FHEE__EEM_Answer__get_attendee_question_answer_value__answer_value', array( $this, 'FHEE__EEM_Answer__get_attendee_question_answer_value__answer_value__pre_fill_question_answers' ), 10, 4 );

		// We don't need the 'powered by EE' in the footer
		add_filter( 'FHEE__EE_Front_Controller__display_registration_footer', '__return_false' );

		// Filter the message shown to users after successful registration
		add_filter( 'FHEE__thank_you_page_overview_template__order_conf_desc', array( $this, 'FHEE__thank_you_page_overview_template__order_conf_desc__thank_you_desc' ), 10, 1 );

		// If the user is not logged in, we don't want to show the ticket selector template
		add_filter( 'FHEE__EE_Ticket_Selector__display_ticket_selector__template_path', array( $this, 'FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__login_template' ), 10, 2 );

		// Register our stylesheet. It is enqueued when we need it
		add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts__register_stylesheet' ) );

		// Display the sign in message before the event location
		add_action( 'AHEE_event_details_before_venue_details', array( $this, 'AHEE_event_details_before_venue_details__display_sign_in_note_before_venue_details' ) );

		// We don't need comments on Events pages
		add_action( 'init', array( $this, 'init__remove_comments_from_events' ), 99 );
		add_filter( 'comments_open', array( $this, 'comments_open__remove_comments_from_events' ), 10, 2 );
		add_filter( 'pings_open', array( $this, 'comments_open__remove_comments_from_events' ), 10, 2 );

		// Add a title to the tickets section just like Event Location has
		add_action( 'AHEE_event_details_before_event_date', array( $this, 'AHEE_event_details_before_event_date__add_ticket_heading' ), 10, 1 );

		// Add a title to the event description section
		add_action( 'AHEE_event_details_after_event_date', array( $this, 'AHEE_event_details_after_event_date__add_event_desc_title' ), 10, 1 );

		// We want the add to iCal link to be a link not a form
		add_filter( 'FHEE__EED_Ical__generate_add_to_iCal_button__iCal_type', array( $this, 'FHEE__EED_Ical__generate_add_to_iCal_button__iCal_type__change_to_link' ), 10, 1 );

		// We want to add the category below the event title
		add_filter( 'FHEE__EEH_Template__display_template__template_path', array( $this, 'FHEE__EEH_Template__display_template__template_path__add_category_to_title' ), 10, 1 );

		// When someone registers for an event, we check their extra info and store it as post meta if needed
		add_filter( 'FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process', array( $this, 'FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_extra_user_meta_on_registration' ), 10, 6 );

		// If the email address attached to a user is the CAS default prefix, then prompt user for extra info
		add_action( 'FHEE__EE_Ticket_Selector__display_ticket_selector__template_path', array( $this, 'FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__show_profile_fields_if_default_email' ), 5, 2 );
		add_action( 'init', array( $this, 'init__intercept_save_of_profile_info' ), 2 );
		// Ensure we can catch our notices
		add_filter( 'query_vars', array( $this, 'query_vars__add_profile_notices_query_var' ), 10, 1 );

		// Add a hidden field for user ID
		add_action( 'AHEE__before_spco_whats_next_buttons', array( $this, 'AHEE__before_spco_whats_next_buttons__add_user_id_hidden_field' ), 10, 3 );
		add_action( 'wp_ajax_process_reg_step', array( $this, 'wp_ajax_process_reg_step__access_user_id_from_reg' ) );
		add_action( 'wp_ajax_nopriv_process_reg_step', array( $this, 'wp_ajax_process_reg_step__access_user_id_from_reg' ) );

		// Modify the message shown at the top of the event-registration page
		add_filter( 'FHEE__registration_page_attendee_information__attendee_information_pg', array( $this, 'FHEE__registration_page_attendee_information__attendee_information_pg__change_message_on_reg_form' ), 10, 1 );

		// We want to show the time left as it'll be weird if someone doesn't know there's a time limit
		add_filter( 'FHEE__registration_page_wrapper_template__display_time_limit', '__return_true' );

		// If there is only one step, we don't need 'Steps' or the heading etc.
		add_filter( 'FHEE__registration_page_wrapper_template__steps_display', array( $this, 'FHEE__registration_page_wrapper_template__steps_display__hide_steps_if_one_step' ), 10, 1 );

		// Capabilities are only set for admins for some reason, we'll need to adjust them for our other use roles
		add_filter( 'FHEE__EE_Capabilities__init_caps_map__caps', array( $this, 'FHEE__EE_Capabilities__init_caps_map__caps__give_caps_we_need' ) );

		// When an event is submitted for review, we need to send an email to the events team
		add_action( 'transition_post_status', array( $this, 'transition_post_status__send_email_when_event_submitted' ), 10, 3 );

		// Set the default Max Registration to 1, not 10. 10 is silly.
		add_filter( 'FHEE__EEH_Template__display_template__template_args', array( $this, 'FHEE__EEH_Template__display_template__template_args__change_default_max_order_to_one' ) );

		// We can't have 'Available Tickets' as a heading...
		add_filter( 'FHEE__ticket_selector_chart_template__table_header_available_tickets', array( $this, 'FHEE__ticket_selector_chart_template__table_header_available_tickets__change_available_tickets_due_to_options_whatever_that_is' ) );

		// Alter button text from 'View Details' to 'Read More'
		add_filter( 'FHEE__EE_Ticket_Selector__display_view_details_btn__btn_text', array( $this, 'FHEE__EE_Ticket_Selector__display_view_details_btn__btn_text__change_view_details_button' ) );

		add_action( 'ubc_ctlt_ee_mods_showing_need_more_info', array( $this, 'ubc_ctlt_ee_mods_showing_need_more_info__load_chosen' ), 10, 2 );

		// Domain mapping causes issues on mapped sites
		add_filter( 'plugins_url', array( $this, 'plugins_url__fix_domain_mapped_urls' ), 0, 3 );
		add_filter( 'plugins_url', array( $this, 'plugins_url__fix_domain_mapped_urls_after_dm_munging' ), 5, 3 );

		// Fix custom fields causing registrations to not happen because of course
		add_filter( 'FHEE__EE_SPCO_Reg_Step_Attendee_Information___save_registration_form_input', array( $this, 'FHEE__EE_SPCO_Reg_Step_Attendee_Information___save_registration_form_input__save_reg_qs' ), 10, 5 );

		// Fix emails not being sent because the 'to' field is empty, because of course it is
		add_filter( 'wp_mail', array( $this, 'wp_mail__fix_to_field_for_event_reg_emails' ), 99, 1 );

		// The attendee meta is empty. Living my best life, right here.
		add_action( 'AHEE__EE_Registration__set_status__after_update', array( $this, 'AHEE__EE_Registration__set_status__after_update__fix_attendee_meta' ), 10, 1 );

	}/* init() */



	/**
	 * When a user registers from the Okanagan site we need to ensure that they are added to the Okanagan
	 * site. Also, it should be made their default site for when the sign in.
	 *
	 * @since 1.0.0
	 *
	 * @param (int) $user_id - The ID of the user that has just been registered
	 * @return null
	 */

	public function user_register__okanagan_user_differences( $user_id ) {

		// Determine if this user has just been created after being on the OK site
		$from_okanagan = $this->is_new_user_from_okanagan();

		if ( false === $from_okanagan ) {
			return;
		}

		// OK, this user is from the Okanagan. Let's add them to the appropriate site.
		// On both verf and prod, the site ID is '4'
		$site_id_to_add_user_to = 4;

		add_user_to_blog( $site_id_to_add_user_to, $user_id, 'subscriber' );

		// Now make this site their default
		update_user_meta( $user_id, 'primary_blog', $site_id_to_add_user_to );

	}/* user_register__okanagan_user_differences() */


	/**
	 * Helper method to determine, from the $_SERVER superglobal - based on the redirect -
	 * if the user is from the okanagan site.
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return (bool) True if the user registered from the Okanagan site. False otherwise
	 */

	public function is_new_user_from_okanagan() {

		// Default to false
		$is_from_okanagan = false;

		$strings_to_match = array(
			'okeventsverf',
			'okevents',
		);

		$haystack = $_SERVER['REQUEST_URI'];

		foreach ( $strings_to_match as $id => $string ) {

			// No match? No dice.
			if ( strpos( $haystack, $string ) === false ) {
				continue;
			}

			// One of our matches has hit the jackpot. WE'RE RICH. Well, the user registered from the OK site at least
			$is_from_okanagan = true;

		}

		return $is_from_okanagan;

	}/* is_new_user_from_okanagan() */


	/**
	 * When a user signs in on the Okanagan site (and they're specifically not going to the admin) then
	 * domain mapping fudges the redirect as it swaps out http -> https because it thinks is_ssl() is true.
	 * We've added a filter to the CAS plugin which allows us to unfudge that.
	 *
	 * @since 1.0.6
	 *
	 * @param (string) $url - The original, possibly fudged, url
	 * @return (string) - If it's an okanagan login, on the front-end, then return http
	 */

	public function ubc_ee_get_original_url__only_http_front_end_okanagan( $url ) {

		$urls_to_unfudge = array(
			// 'https://ok.eventsverf.ctlt.ubc.ca',
			// 'https://okeventsverf.ctlt.ubc.ca',
			'https://ok.events.ctlt.ubc.ca',
			'https://okevents.ctlt.ubc.ca',
			'https://ctlevents.ok.ubc.ca',
		);

		foreach ( $urls_to_unfudge as $id => $fudged_url ) {

			if ( untrailingslashit( $url ) !== $fudged_url ) {
				continue;
			}

			// OK this is a fudged URL, we need to ensure it is http on the front-end
			$url = str_replace( 'https://', 'http://', $url );

		}

		return $url;

	}/* ubc_ee_get_original_url__only_http_front_end_okanagan() */


	/**
	 * Adjust the 'price' column in the ticket selection table. If there's a non-zero price, then
	 * just leave it as-is. However, if it's a free event, we just want an empty string
	 *
	 * @since 1.0.0
	 *
	 * @param string $default - The default text passed in i.e. 'Price'
	 * @param int $event_id - The ID of the event we're adjusting this table heading for
	 * @return string - Empty string if free event, or whatever is passed in if it isn't
	 */

	public function FHEE__ticket_selector_chart_template__table_header_price__adjust_price_header( $default = false, $event_id = false ) {

		// If we can't access the tickets, just bail
		if ( empty( $event_id ) || empty( static::$passed_template_args ) || ! isset( static::$passed_template_args['tickets'] ) ) {
			return $default;
		}

		// Grab the tickets
		$tickets = static::$passed_template_args['tickets'];

		if ( empty( $tickets ) || ! is_array( $tickets ) ) {
			return $default;
		}

		// We need to work out if any of the tickets do not have a zero-price. If they don't,
		// we just return the default. If all of the tickets are zero price, we don't need
		// the heading
		$all_tickets_free = true;

		foreach ( $tickets as $ticket_id => $ticket_object ) {

			// Ensure this actually is a ticket object
			if ( ! is_a( $ticket_object, 'EE_Ticket' ) ) {
				continue;
			}

			$is_free = $ticket_object->is_free();

			if ( false === $is_free ) {
				$all_tickets_free = false;
				break;
			}
		}

		// OK, all tickets are free, so we don't need the heading
		if ( true === $all_tickets_free ) {
			return '';
		}

		// All tickets are not free, so the default is appropriate
		return $default;

	}/* FHEE__ticket_selector_chart_template__table_header_price__adjust_price_header' )() */



	/**
	 * Adjust the 'Qty.' column in the ticket selection table. If we only have one ticket per-person
	 * available then we simply don't need the heading.
	 *
	 * @since 1.0.0
	 *
	 * @param string $default - The default text passed in i.e. 'Qty*'
	 * @param int $event_id - The ID of the event we're adjusting this table heading for
	 * @return string - Empty string if there's only one ticket allowed or whatever is passed in if it isn't
	 */

	public function FHEE__ticket_selector_chart_template__table_header_qty__adjust_qty( $default = false, $event_id = false ) {

		if ( empty( $event_id ) || empty( static::$passed_template_args ) || ! isset( static::$passed_template_args['tickets'] ) ) {
			return $default;
		}

		// If the max number of tickets available is more than 1, we just need to return the default
		$max_tickets_per_order = EEM_Event::instance()->get_one_by_ID( $event_id )->get( 'EVT_additional_limit' );

		if ( absint( $max_tickets_per_order > 1 ) ) {
			return $default;
		}

		// If we have multiple tickets, the heading should be 'Ticket type'
		$tickets = static::$passed_template_args['tickets'];

		if ( empty( $tickets ) || ! is_array( $tickets ) || count( $tickets ) < 2 ) {
			// OK, we don't need this heading, so just have an empty string
			return '';
		}

		// We have multiple tickets, and max is only one, so;
		return 'Ticket Type';

	}/* FHEE__ticket_selector_chart_template__table_header_qty__adjust_qty() */


	/**
	 * Adjust the output of the ticket price to 'Free' if the ticket is...free
	 *
	 * @since 1.0.0
	 *
	 * @param int $amount - The price of the ticket
	 * @param bool $return_raw - Whether we are returning this raw
	 * @return int|string - 'Free' if the ticket is free, or the default passed in otherwise
	 */

	public function FHEE__EEH_Template__format_currency__amount__adjust_free_price_output( $amount, $return_raw ) {

		// Try to ensure we have some sort of data consistency
		$amount = (float) $amount;

		// If it's free, we say so
		if ( 0 === $amount || '0.00' === $amount || 0.00 === $amount ) {
			return __( 'Free', 'ubc-ctlt-event-espresso-modifications' );
		}
		return $amount;

	}/* FHEE__EEH_Template__format_currency__amount__adjust_free_price_output() */


	/**
	 * If the ticket is free, don't add '/ ticket' to the 'Free' we changed it to above
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $default - the default string to append to the price ('/ ticket')
	 * @return (string) An empty string
	 */

	public function FHEE__ticket_selector_chart_template__per_ticket_text__no_slash_ticket( $default ) {

		return '';

	}/* FHEE__ticket_selector_chart_template__per_ticket_text__no_slash_ticket() */


	/**
	 * When EE loads a template it can pass arguments to it. We need a nice way to get access to the
	 * tickets for this event when we're adjusting the table headers, so this class method hooks into the
	 * FHEE__EEH_Template__display_template__template_args filter provided by EE core and sets the passed
	 * args as a class argument here, so we can access them.
	 *
	 * @since 1.0.0
	 *
	 * @param array $template_args - The passed template args
	 * @return array $template_args - unmodified template args
	 */

	public function FHEE__EEH_Template__display_template__template_args__set_class_properties_for_args( $template_args ) {

		// We just want to set this to a class variable so we're able to access these args elsehwere
		static::$passed_template_args = $template_args;

		// Return unmodified
		return $template_args;

	}/* FHEE__EEH_Template__display_template__template_args__set_class_properties_for_args() */


	/**
	 * When EE loads a ticket it has a filter at the top which allows us to overwrite the whole
	 * row. We don't want to do that as it is unnecessary. This method allows us to set a class
	 * property of the price so that we're able to hook in later and determine if the ticket is
	 * free. If it is, we can change the layout/content of the ticket
	 *
	 * @since 1.0.0
	 *
	 * @param (mixed) $multiple An entirely unnecessary list of 9 arguments
	 * @return bool false - we return false as we don't want to overwrite the whole row
	 */

	public function FHEE__ticket_selector_chart_template__do_ticket_inside_row__set_class_properties_for_args( $default, $ticket, $max, $min, $required_ticket_sold_out, $ticket_price, $ticket_bundle, $ticket_status, $status_class ) {

		static::$ticket_price = $ticket_price;

		remove_filter( 'FHEE__ticket_selector_chart_template__per_ticket_text', array( $this, 'FHEE__ticket_selector_chart_template__per_ticket_text__no_slash_ticket' ), 10, 1 );

		// If the price is 0, we need to add a filter
		if ( 0 === $ticket_price ) {
			add_filter( 'FHEE__ticket_selector_chart_template__per_ticket_text', array( $this, 'FHEE__ticket_selector_chart_template__per_ticket_text__no_slash_ticket' ), 10, 1 );
		}

		return $default;

	}/* FHEE__ticket_selector_chart_template__do_ticket_inside_row__set_class_properties_for_args() */


	/**
	 * Add extra user profile fields for people to fill in which should help people from
	 * having to reproduce their effort every time they sign up for an event
	 *
	 * @since 1.0.0
	 *
	 * @param (object) $user - The current user object
	 * @return null
	 */

	public function show_edit_user_profile__add_events_fields( $user ) {

		?>
		<h3>UBC Details</h3>
		<p>Saving information for these fields will allow you to not have to choose them for every event you register.</p>
		<table class="form-table">
		<?php

		// Grab saved data if exists
		$fac_school_college 	= get_the_author_meta( 'fac_school_college', $user->ID );
		$department 			= get_the_author_meta( 'department', $user->ID );
		$institution 			= get_the_author_meta( 'institution', $user->ID );
		$attending_as 			= get_the_author_meta( 'attending_as', $user->ID );

		$additional_form_fields = static::get_ubc_reg_form_fields_by_question_group_id( static::$question_group_id_for_our_custom_questions );
		$stuff = array();
		foreach ( $additional_form_fields as $question_id => $field_details ) :

			switch ( $question_id ) {

				case 20:
					$field_to_check = $fac_school_college;
					break;

				case 21:
					$field_to_check = $department;
					break;

				case 22:
					$field_to_check = $institution;
					break;

				case 23:
					$field_to_check = $attending_as;
					break;

				default:
					$field_to_check = $fac_school_college;
					break;
			}
		?>

		<tr>
			<th><label id="<?php echo esc_html( static::$question_id_user_meta_map[ $question_id ] ); ?>" for="<?php echo esc_html( static::$question_id_user_meta_map[ $question_id ] ); ?>"><?php echo esc_html( $field_details['label'] ); ?></label></th>
			<td>
				<select id="<?php echo esc_html( static::$question_id_user_meta_map[ $question_id ] ); ?>" name="<?php echo esc_html( static::$question_id_user_meta_map[ $question_id ] ); ?>" aria-required="true" required>
					<?php foreach ( $field_details['options'] as $option_key => $option_value ) : ?>
						<option <?php selected( htmlspecialchars( html_entity_decode( $field_to_check ) ), htmlspecialchars( html_entity_decode( $option_value ) ) ); ?> value="<?php echo esc_html( $option_value ); ?>"><?php echo esc_html( $option_value ); ?></option>
					<?php endforeach; ?>
				</select>
			</td>
		</tr>

	<?php endforeach; ?>

	</table>

	<?php

	}/* show_edit_user_profile__add_events_fields() */


	/**
	 * Save the extra user profile fields
	 *
	 * @since 1.0.0
	 *
	 * @param (int) $user_id - The ID of the user to which we are saving
	 * @return null
	 */

	public function personal_options_update__save_events_fields( $user_id ) {

		update_user_meta( $user_id, 'fac_school_college', 	sanitize_text_field( $_POST['fac_school_college'] ) );
		update_user_meta( $user_id, 'department', 			sanitize_text_field( $_POST['department'] ) );
		update_user_meta( $user_id, 'institution', 			sanitize_text_field( $_POST['institution'] ) );
		update_user_meta( $user_id, 'attending_as', 		sanitize_text_field( $_POST['attending_as'] ) );

	}/* personal_options_update__save_events_fields() */


	/**
	 * Pre-fill answers to questions for which we have the answer stored in user meta.
	 * This is most useful for the email address and fac/dept questions
	 *
	 * @since 1.0.0
	 *
	 * @param (mixed) $value - The question answer
	 * @param (object) $registration - An EE Registration object
	 * @param (int) $question_id - The question ID to which we may want the answer pre-filled
	 * @return (string) The answer if we have any
	 */

	public function FHEE__EEM_Answer__get_attendee_question_answer_value__answer_value__pre_fill_question_answers( $value, $registration, $question_id, $question_system_id ) {

		// Pointless if the user isn't logged in.
		if ( ! is_user_logged_in() ) {
			return $value;
		}

		$user_id 	= get_current_user_id();
		$user_data 	= get_userdata( absint( $user_id ) );

		// Some sanity checking
		if ( ! $user_data || ! is_a( $user_data, 'WP_User' ) ) {
			return $value;
		}

		// Default fields we need to fetch are first name, last name and email.
		$first_name 		= ucfirst( sanitize_text_field( $user_data->first_name ) );
		$last_name 			= ucfirst( sanitize_text_field( $user_data->last_name ) );
		$email_address 		= $user_data->user_email;

		// Extra user meta fields we add for UBC bits and pieces
		$institution 		= $user_data->institution;
		$fac_school_college	= $user_data->fac_school_college;
		$department 		= $user_data->department;
		$attending_as 		= $user_data->attending_as;

		switch ( $question_id ) {

			case 1:
				// First Name
				return $first_name;
				break;

			case 2:
				// Last name
				return $last_name;
				break;

			case 3:
				// email
				return $email_address;
				break;

			case 20:
				// Faculty/School/College
				return $fac_school_college;
				break;

			case 21:
				// Dept
				return $department;
				break;

			case 22:
				// Institution
				return $institution;
				break;

			case 23:
				// Attending As
				return $attending_as;
				break;

			default:
				return $value;
				break;

		}

		return $answer;

	}/* FHEE__EEM_Answer__get_attendee_question_answer_value__answer_value__pre_fill_question_answers() */


	/**
	 * The 'thank you' message shown to users includes words like 'purchase'. We don't want that, so let's
	 * have our custom message shown here.
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $message - The message shown
	 * @return (string) A modified message
	 */

	public function FHEE__thank_you_page_overview_template__order_conf_desc__thank_you_desc( $message ) {

		return __( 'Your registration has been <strong>successful</strong>. The details are shown below. Please <strong>check your email</strong> for registration confirmation.', 'ubc-ctlt-event-espresso-modifications' );

	}/* FHEE__thank_you_page_overview_template__order_conf_desc__thank_you_desc() */


	/**
	 * If a user is signed in, we should not show the ticket selector and associated form. Instead we should
	 * show a message to the user that they must sign in, provide a link to sign in and return to the event
	 * page
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $template_path - The path to the template. This overwrites the ticket_selector_chart.template.php
	 * @param (object) $event - The event object of the event we're currently on
	 * @return (string) a path to our custom you must log in template
	 */

	public function FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__login_template( $template_path, $event ) {

		// If the user is signed in already, display the ticket selector
		if ( is_user_logged_in() ) {
			return $template_path;
		}

		// Pass back the path to the template
		return plugin_dir_path( __FILE__ ) . 'templates/you-must-sign-in.php';

	}/* FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__login_template() */


	/**
	 * Register the stylesheet so we can enqueue it as and when we need
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function wp_enqueue_scripts__register_stylesheet() {

		if ( defined( 'CTLT_ENVIRONMENT' ) && 'LOCAL' === CTLT_ENVIRONMENT ) {
			$asset_url = WPMU_PLUGIN_URL . '/ubc-ctlt-event-espresso-modifications/assets/css/ubc-ctlt-event-espresso-modifications.css';
		} else {
			$asset_url = plugins_url( 'assets/css/ubc-ctlt-event-espresso-modifications.css', __FILE__ );
		}

		wp_register_style( 'ubc-ctlt-event-espresso-modifications', $asset_url );
		wp_enqueue_style( 'ubc-ctlt-event-espresso-modifications' );

	}/* wp_enqueue_scripts__register_stylesheet() */


	/**
	 * Also display a notice about registration required after the event description, just so it's obvious
	 *
	 * @since 1.0.0
	 *
	 * @param (object) $post - the post object
	 * @return null
	 */

	public function AHEE_event_details_before_venue_details__display_sign_in_note_before_venue_details( $post ) {

		// Only load this template if a user is not signed in
		if ( is_user_logged_in() ) {
			return;
		}

		// The main use for this template is to replace the ticket selector. And that passes through some
		// template args, so we have to do likewise here.
		$template_args = array();
		$template_args['EVT_ID'] = $post->ID;

		// Load the template and pass it our args
		echo wp_kses_post( EEH_Template::locate_template( plugin_dir_path( __FILE__ ) . 'templates/you-must-sign-in.php', $template_args ) );

	}/* AHEE_event_details_before_venue_details__display_sign_in_note_before_venue_details() */


	/**
	 * Remove comments from the events post type
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function init__remove_comments_from_events() {

		remove_post_type_support( 'espresso_events', 'comments' );
		remove_post_type_support( 'espresso_events', 'trackbacks' );

	}/* init__remove_comments_from_events() */


	/**
	 * Ensure comments aren't open for events
	 *
	 * @since 1.0.0
	 *
	 * @param (bool) $open - Are comments open for this post
	 * @param (int) $post_id - the ID of the post for which we are checking if comments are open
	 * @return (bool) Whether comments should be open for this $post_id
	 */

	public function comments_open__remove_comments_from_events( $open, $post_id ) {

		$post_type = get_post_type( $post_id );

		if ( 'espresso_events' !== $post_type ) {
			return $open;
		}

		return false;

	}/* comments_open__remove_comments_from_events() */


	/**
	 * The 'Event Location' section has a heading and an icon. The ticketing section doesn't.
	 * This method adds a heading to make the display of the events much more consistent.
	 *
	 * @since 1.0.0
	 *
	 * @param (object) $post - The current $post object
	 * @return null
	 */

	public function AHEE_event_details_before_event_date__add_ticket_heading( $post ) {

		$template_args = array();
		$template_args['post'] = $post;

		echo wp_kses_post( EEH_Template::locate_template( plugin_dir_path( __FILE__ ) . 'templates/event-date-time-heading.php', $template_args ) );

	}/* AHEE_event_details_before_event_date__add_ticket_heading() */


	/**
	 * The Event Description section doesn't have any sort of title or anything and it just
	 * kind of...sits there. Let's add a title so it fits in with date/time and location sections
	 *
	 * @since 1.0.0
	 *
	 * @param (object) $post - A WP Post Object of the current post
	 * @return null
	 */

	public function AHEE_event_details_after_event_date__add_event_desc_title( $post ) {

		$template_args = array();
		$template_args['post'] = $post;

		echo wp_kses_post( EEH_Template::locate_template( plugin_dir_path( __FILE__ ) . 'templates/event-description-heading.php', $template_args ) );

	}/* AHEE_event_details_after_event_date__add_event_desc_title() */


	/**
	 * Change the link to download to iCal to be a link and not a form
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $type - The type of iCal link
	 * @return (string) we want an icon
	 */

	public function FHEE__EED_Ical__generate_add_to_iCal_button__iCal_type__change_to_link( $type ) {

		return 'icon';

	}/* FHEE__EED_Ical__generate_add_to_iCal_button__iCal_type__change_to_link() */


	/**
	 * We need to add the category to each event's description. Sadly, there's no action or filter
	 * given to us where we need it, so we need to replace the actual template
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $template_path - A string of absolute paths
	 * @return (string) modified string
	 */

	public function FHEE__EEH_Template__display_template__template_path__add_category_to_title( $template_path ) {

		// Ensure we're only doing what we need to do for the right template
		// There's no filter or anything in the actual title (and I don't want to add something to the_title
		// as that can only end badly for performance)
		// We need to see when the tickets template is being loaded and then output something before it
		$string_to_look_for = 'content-espresso_events-tickets.php';

		if ( false === strpos( $template_path, $string_to_look_for ) ) {
			return $template_path;
		}

		global $post;

		if ( ! $post || ! is_a( $post, 'WP_Post' ) ) {
			return $template_path;
		}

		$template_args = array();
		$template_args['post'] = $post;

		$categories = get_the_terms( $post->ID, 'espresso_event_categories' );

		if ( ! $categories || ! is_array( $categories ) ) {
			return $template_path;
		}

		$categories_string = '';

		foreach ( $categories as $category ) {

			$term_link = get_term_link( $category );
			if ( is_wp_error( $term_link ) ) {
				continue;
			}
			$categories_string .= '<span class="indiv-term-link"><a href="' . esc_url( $term_link ) . '">' . esc_html( $category->name ) . '</a></span> ';
		}

		$template_args['categories_string'] = $categories_string;

		// Location of our additional template
		$extra_template = plugin_dir_path( __FILE__ ) . 'templates/event-categories.php';

		echo wp_kses_post( EEH_Template::locate_template( $extra_template, $template_args ) );

		return $template_path;

	}/* FHEE__EEH_Template__display_template__template_path__add_category_to_title() */


	/**
	 * When a user registers for an event, we check their user meta to see if we have stored
	 * information for them for our extra reg fields. If we don't then we store it for them.
	 *
	 * @since 1.0.0
	 *
	 * @param (bool) $process_reg - Whether to process the registration or not
	 * @param (int) $att_number - The attendance number
	 * @param (object) $registration - The full registration object
	 * @param (array) $registrations - All of the registrations from this transaction
	 * @param (array) $valid_data - The validated data passed to _process_registrations()
	 * @param (object) $attendee_object - The full EE_SPCO_Reg_Step_Attendee_Information object
	 * @return (bool) Whether to carry on processing the registration or not
	 */

	public function FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_extra_user_meta_on_registration( $process_reg, $att_nmbr, $registration, $registrations, $valid_data, $attendee_object ) {

		// First we look for some useful things in $valid_data. Sadly this is difficult to cherry pick
		// out from EE, so we have to hard code some strings.
		// @todo See if we can abstract this nonsense
		//
		// $valid_data should be an associative array of arrays. The first key will probably be
		// 'default_hidden_inputs' which should contain bits and pieces about the form itself
		// There should also be another key to this array (which is a regi id :
		// $reg_url_link = $registration->reg_url_link(); $valid_data[ $reg_url_link ]
		// which contains an array of extra info, i.e.
		// [fac-dept-inst-attending-as] => Array
		//	(
		//		[20] => Architecture and Landscape Architecture
		//		[21] => Access and Diversity
		//		[22] => UBC
		//		[23] => Faculty
		//	)
		//	The integers in that array are the Question IDs
		//	We need to get these data out and save it if we need to

		// Check if we have data for this user
		// Need an object. This gets set in wp_ajax_process_reg_step__access_user_id_from_reg as we had to append it to the form
		$user_id = static::$user_id;

		$user_data = get_userdata( $user_id );

		if ( empty( $user_data ) || ! is_a( $user_data, 'WP_User' ) ) {
			return $process_reg;
		}

		// These are all the potential user meta fields we need to check
		$fields_to_check = static::$question_id_user_meta_map;

		// If the fields are empty, we try and update them with what we've just received from the reg
		$fields_to_update = array();

		// Do we only want to update empty fields?
		$only_update_empty_fields = false;

		// Work out which fields we need to update (i.e. empty fields)
		foreach ( $fields_to_check as $question_id => $field_name ) {

			if ( $only_update_empty_fields ) {

				$this_field_data = $user_data->$field_name;

				if ( empty( $this_field_data ) ) {
					$fields_to_update[ $question_id ] = $field_name;
				}
			} else {
				$fields_to_update[ $question_id ] = $field_name;
			}
		}

		// OK, now we have a list of fields for which to update. But if it's empty, bail
		if ( empty( $fields_to_update ) ) {
			return $process_reg;
		}

		$reg_url_link = $registration->reg_url_link(); // this is the key for the array

		// Sad panda that this is hardcoded
		$question_group_field_key = 'fac-dept-inst-attending-as';

		$submitted_data = $valid_data[ $reg_url_link ][ $question_group_field_key ];

		if ( ! is_array( $submitted_data ) ) {
			return $process_reg;
		}

		foreach ( $submitted_data as $question_id => $submitted_value ) {

			// If this question ID isn't in the list for which we need to update, ignore
			if ( ! array_key_exists( $question_id, $fields_to_update ) ) {
				continue;
			}

			update_user_meta( absint( $user_id ), static::$question_id_user_meta_map[ $question_id ], esc_html( $submitted_value ) );

		}

		return $process_reg;

	}/* FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_extra_user_meta_on_registration() */


	/**
	 * If a user is signed in, but we don't have a proper email address from them (i.e. their email address)
	 * has the CAS default suffix, then we don't let them register for an event, we require their details
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $template_path - The path to the template. This overwrites the ticket_selector_chart.template.php
	 * @param (object) $event - The event object of the event we're currently on
	 * @return (string) a path to our custom you must log in template
	 */

	public function FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__show_profile_fields_if_default_email( $template_path, $event ) {

		// If the user is not signed in already, defer to what we would normally have (this gets hoked into later to add a message template)
		if ( ! is_user_logged_in() ) {
			return $template_path;
		}

		// Now check if the currently logged in user's email address has the default cas suffix
		$cas_options = get_site_option( 'cas_authentication_options' );
		if ( ! $cas_options || empty( $cas_options ) || ! is_array( $cas_options ) || ! isset( $cas_options['email_suffix'] ) ) {
			return $template_path;
		}

		// Check if the user's email address contains the CAS suffix, if it doesn't, revert to your normal programming
		$email_suffix = $cas_options['email_suffix'];

		$user_id 		= get_current_user_id();
		$user_data 		= get_userdata( absint( $user_id ) );
		$email_address 	= $user_data->user_email;

		if ( false === strpos( $email_address, $email_suffix ) ) {
			return $template_path;
		}

		add_filter( 'body_class', array( $this, 'body_class__hide_event_details' ), 10, 1 );

		do_action( 'ubc_ctlt_ee_mods_showing_need_more_info', $user_id, $user_data );

		// Pass back the path to the template
		return plugin_dir_path( __FILE__ ) . 'templates/we-need-more-info-about-you.php';

	}/* FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__show_profile_fields_if_default_email() */


	/**
	 * If the user hasn't provided a valid email address (i.e. it's still the CAS default), we need
	 * to hide the event details form when they're on an event page. We show a form for them to fill
	 * in, but if the other fields on the page it could be confusing.
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $body_classes - Currently set body classes
	 * @return (array) Modified body classes
	 */

	public function body_class__hide_event_details( $body_classes ) {

		$body_classes[] = 'ctlt-hide-event-details';

		return $body_classes;

	}/* body_class__hide_event_details() */


	/**
	 * Hook in early and see if the save profile form stuff has been initiated. If so,
	 * validate and save then return the user back to where they were but this time to
	 * register for the event
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function init__intercept_save_of_profile_info() {

		// Check we're submitting
		if ( ! isset( $_POST['ubc_ctlt_extra_profile_info'] ) ) {
			return;
		}

		// Verify our nonce or bail
		if ( ! wp_verify_nonce( $_POST['ubc_ctlt_extra_profile_info'], 'ubc_ctlt_extra_profile_info' ) ) {
			return;
		}

		// Ensure we can go back to whence we came
		$referer = esc_url( $_POST['_wp_http_referer'] );

		// Sanitize our data
		$user_email = sanitize_email( $_POST['user_email'] );
		$user_fname = sanitize_text_field( $_POST['user_fname'] );
		$user_lname = sanitize_text_field( $_POST['user_lname'] );

		// @todo: remove hardcodedness of these questions
		$fac_school_college 	= ( isset( $_POST['question_20'] ) ) ? sanitize_text_field( $_POST['question_20'] ) : false;
		$department 			= ( isset( $_POST['question_21'] ) ) ? sanitize_text_field( $_POST['question_21'] ) : false;
		$institution 			= ( isset( $_POST['question_22'] ) ) ? sanitize_text_field( $_POST['question_22'] ) : false;
		$attending_as 			= ( isset( $_POST['question_23'] ) ) ? sanitize_text_field( $_POST['question_23'] ) : false;

		// Validate data.
		$notices = '';

		// We need an email
		if ( empty( $user_email ) ) {
			$notices = 'email-required';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		// if email exists, inform the user
		if ( is_email( $user_email ) && email_exists( $user_email ) ) {
			$notices = 'email-exists';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		if ( empty( $user_fname ) || empty( $user_lname ) ) {
			$notices = 'full-name-required';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		if ( $fac_school_college && 'Choose Faculty, School or College' === $fac_school_college ) {
			$notices = 'faculty-required';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		if ( $department && 'Choose Department' === $department ) {
			$notices = 'department-required';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		if ( $institution && 'Choose Institution' === $institution ) {
			$notices = 'institution-required';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		if ( $attending_as && 'Choose Attending As' === $attending_as ) {
			$notices = 'attending-as-required';
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;
		}

		// OK, now we need to update this user
		$user_id = get_current_user_id();

		$updates = array(
			'ID' => absint( $user_id ),
			'user_email' => $user_email,
			'first_name' => $user_fname,
			'last_name' => $user_lname,
		);

		$updated = wp_update_user( $updates );

		if ( is_wp_error( $user_id ) ) {

			$notices = $updated->get_error_message();
			wp_safe_redirect( add_query_arg( 'ubc_ctlt_notices', $notices, $referer ) );
			exit;

		} else {

			if ( $fac_school_college ) {
				update_user_meta( $user_id, 'fac_school_college', esc_html( $fac_school_college ) );
			}

			if ( $department ) {
				update_user_meta( $user_id, 'department', esc_html( $department ) );
			}

			if ( $institution ) {
				update_user_meta( $user_id, 'institution', esc_html( $institution ) );
			}

			if ( $attending_as ) {
				update_user_meta( $user_id, 'attending_as', esc_html( $attending_as ) );
			}

			wp_safe_redirect( remove_query_arg( 'ubc_ctlt_notices', $referer ) );
			exit;
		}

	}/* init__intercept_save_of_profile_info() */


	/**
	 * Add an extra query var for use when submitting the profile info. Added in init__intercept_save_of_profile_info()
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $vars - Currently existing query vars
	 * @return (array) Ammended query vars
	 */

	public function query_vars__add_profile_notices_query_var( $vars ) {

		$vars[] = 'ubc_ctlt_notices';

		return $vars;

	}/* query_vars__add_profile_notices_query_var() */


	/**
	 * Currently it doesn't look like there's helper functions with EE to get registration questions
	 * easily. We want to display the 'Fac Dept Inst Attending As' question group on both the front-end
	 * when a user hasn't filled in the info before and also in the user profile so they're able to
	 * easily edit this information.
	 *
	 * This method provides a way to get at tho fields so they only have to be input in once place and
	 * can be used elsewhere
	 *
	 * @since 1.0.0
	 *
	 * @todo This only works for dropdowns at the moment (all we use, should abstract to other types)
	 * @param (array) $question_ids - An array of question IDs of fields to get
	 * @return (string) An HTML string of form fields
	 */

	public static function get_ubc_reg_form_fields_by_question_ids( $question_ids = array() ) {

		// Default to all the questions
		if ( ! isset( $question_ids ) || empty( $question_ids ) || ! is_array( $question_ids ) ) {
			$question_ids = array( 20, 21, 23, 23 );
		}

		global $wpdb;
		$question_table 		= $wpdb->prefix . 'esp_question';
		$question_option_table 	= $wpdb->prefix . 'esp_question_option'; // Is there a DEFINE for this?

		// The start of our data store
		$field_data = array();

		// For each of the question IDs we need to get the possible options for it.
		foreach ( $question_ids as $key => $question_id ) {

			$this_field = array();

			// First get the 'label'
			$question_label = $wpdb->get_var(
				$wpdb->prepare(
					"SELECT QST_display_text FROM $question_table WHERE `QST_ID` = %d",
					$question_id
				)
			);

			$this_field['label'] = $question_label;

			// Now get the possible options
			$question_options = $wpdb->get_col(
				$wpdb->prepare(
					"SELECT QSO_value FROM $question_option_table WHERE `QST_ID` = %d AND `QSO_deleted` = %d ORDER BY `QSO_order`",
					$question_id,
					0
				)
			);

			$this_field['options'] = $question_options;

			$field_data[ $question_id ] = $this_field;

		}

		return $field_data;

	}/* get_ubc_reg_form_fields() */


	public static function get_ubc_reg_form_fields_by_question_group_id( $group_id = false ) {

		if ( ! $group_id ) {
			return false;
		}

		$group_id = absint( $group_id );

		global $wpdb;
		$question_option_table = $wpdb->prefix . 'esp_question_group_question'; // Is there a DEFINE for this?

		// Get a list of question IDs for this group
		$question_ids = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT QST_ID FROM $question_option_table WHERE `QSG_ID` = %d",
				$group_id
			)
		);

		// $question_ids should now be an array of integers
		if ( ! $question_ids || ! is_array( $question_ids ) || empty( $question_ids ) ) {
			return false;
		}

		return static::get_ubc_reg_form_fields_by_question_ids( $question_ids );

	}/* get_ubc_reg_form_fields_by_question_group_id() */


	/**
	 * We need to add the user ID to the POST submitted when a user registers because for some reason
	 * it's being cleared. We later access this in the AJAX event wp_ajax_process_reg_step__access_user_id_from_reg Callback
	 * which then sets it to a class variable which we're then able to access in
	 * FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_extra_user_meta_on_registration
	 * to be able to add some user meta
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $slug - The path to the current step in the form
	 * @param (string) $next_step_slug - The path to the next step in the form
	 * @param (object) $checkout - The entire checkout object
	 * @return null;
	 */

	public function AHEE__before_spco_whats_next_buttons__add_user_id_hidden_field( $slug, $next_step_slug, $checkout ) {

		if ( ! is_user_logged_in() ) {
			return;
		}
		$user_id = get_current_user_id();

		$user_id_field = '<input type="hidden" readonly value="' . absint( $user_id ) . '" name="ubc_ctlt_user_id" id="ubc_ctlt_user_id" />';

		$output = apply_filters( 'ubc_ee_add_output_to_reg_form', $user_id_field );

		echo $output;

	}/* AHEE__before_spco_whats_next_buttons__add_user_id_hidden_field() */


	/**
	 * HOLY HACK BATMAN
	 * For some ridiculous reason the user object is getting removed when we submit. So,
	 * we have to hook into the AJAX action and store the user ID that is submitted as a new
	 * hidden field to a class variable. We can then access it when we're setting the user meta
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function wp_ajax_process_reg_step__access_user_id_from_reg() {

		if ( ! isset( $_POST ) || empty( $_POST ) || ! isset( $_POST['ubc_ctlt_user_id'] ) ) {
			return;
		}

		$user_id = absint( $_POST['ubc_ctlt_user_id'] );

		static::$user_id = $user_id;

	}/* wp_ajax_process_reg_step__access_user_id_from_reg() */


	/**
	 * Modify the message shown at the top of the 'Event Registration' page with more unseful info
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $message - The default message shown
	 * @return (string) The modified message
	 */

	public function FHEE__registration_page_attendee_information__attendee_information_pg__change_message_on_reg_form( $message ) {

		return __( 'Please provide the following information. All fields are required. We will save the information you provide to your profile so you will not have to complete it next time.', 'ubc-ctlt-event-espresso-modifications' );

	}/* FHEE__registration_page_attendee_information__attendee_information_pg__change_message_on_reg_form() */


	public function FHEE__registration_page_wrapper_template__steps_display__hide_steps_if_one_step( $show_steps ) {

		if ( ! isset( static::$passed_template_args ) || ! isset( static::$passed_template_args['reg_steps'] ) ) {
			return $show_steps;
		}

		// 'finalize_registration' is classified as a step, so, the $total_steps will always be one more than the number of actual steps
		$total_steps = count( static::$passed_template_args['reg_steps'] ) - 1;

		// If we only really have one step, we don't need all this guff
		if ( 1 === $total_steps ) {
			return false;
		}

		return $show_steps;

	}/* FHEE__registration_page_wrapper_template__steps_display__hide_steps_if_one_step() */


	/**
	 * We have fairly specific capabilty needs for users on our platform, so set them here
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $caps - Capabilities for roles
	 * @return (array) modified capabilities
	 */

	public function FHEE__EE_Capabilities__init_caps_map__caps__give_caps_we_need( $caps ) {

		// Espresso Event Managers can submit (but not publish) events, venues, registrations etc.
		$caps['espresso_event_manager'] = array(
			'ee_read_ee' => 'ee_read_ee',
			'ee_read_registration' => 'ee_read_registration',
			'ee_read_registrations' => 'ee_read_registrations',
			'ee_read_others_registrations' => 'ee_read_others_registrations',
			'ee_read_private_events' => 'ee_read_private_events',
			'ee_read_others_events' => 'ee_read_others_events',
			'ee_read_event' => 'ee_read_event',
			'ee_read_events' => 'ee_read_events',
			'ee_edit_event' => 'ee_edit_event',
			'ee_edit_events' => 'ee_edit_events',
			'ee_edit_published_events' => 'ee_edit_published_events',
			'ee_edit_others_events' => 'ee_edit_others_events',
			'ee_assign_event_category' => 'ee_assign_event_category',
			'ee_read_venue' => 'ee_read_venue',
			'ee_read_venues' => 'ee_read_venues',
			'ee_read_others_venues' => 'ee_read_others_venues',
			'ee_assign_venue_category' => 'ee_assign_venue_category',
			'espresso_event_manager' => 'espresso_event_manager',
		);

		// espresso_group_admin can view registrations and that's about it
		$caps['espresso_group_admin'] = array(
			'ee_read_ee' => 'ee_read_ee',
			'espresso_group_admin' => 'espresso_group_admin',
			'ee_read_registration' => 'ee_read_registration',
			'ee_read_registrations' => 'ee_read_registrations',
			'ee_read_others_registrations' => 'ee_read_others_registrations',
		);

		return $caps;

	}/* FHEE__EE_Capabilities__init_caps_map__caps__give_caps_we_need() */


	/**
	 * Change the default limit to '1' for the Maximum number of tickets allowed per order for an event
	 *
	 * @since 1.0.0
	 *
	 * @param array $template_args - The passed template args
	 * @return array $template_args - unmodified template args
	 */

	public function FHEE__EEH_Template__display_template__template_args__change_default_max_order_to_one( $template_args ) {

		if ( is_array( $template_args ) && ! isset( $template_args['additional_limit'] ) ) {
			return $template_args;
		}

		// This will mean we can never have 10 for the limit though, but this should be fine
		if ( is_array( $template_args ) && 10 === $template_args['additional_limit'] ) {
			$template_args['additional_limit'] = 1;
		}

		return $template_args;

	}/* FHEE__EEH_Template__display_template__template_args__change_default_max_order_to_one() */


	/**
	 * When an event is submitted by a non-admin, it's submitted for review. We'll send an email to
	 * the events team when this happens
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $new_status - The status being transitioned TO
	 * @param (string) $old_status - The status being transitioned FROM
	 * @param (object) $post - The WP_Post object
	 * @return null
	 */

	public function transition_post_status__send_email_when_event_submitted( $new_status, $old_status, $post ) {

		if ( 'pending' !== $new_status ) {
			return;
		}

		// If this is an edit of the same post, then the old and new will be the same, so we don't need to send again
		if ( $new_status === $old_status ) {
			return;
		}

		// Sanity check the post
		if ( ! isset( $post ) || empty( $post ) || ! is_a( $post, 'WP_Post' ) ) {
			return;
		}

		$post_id = absint( $post->ID );

		// This variable will be used in the email if there are any additional messages we wish to show
		$messages = array();

		// Get the post edit link so it's nice and easy in the email
		$post_edit_link = get_edit_post_link( $post_id );

		// Post Title so we can see what event it is more easily
		$post_title = $post->post_title;

		if ( empty( $post_title ) ) {
			$messages[] = 'No title was entered.';
		}

		// Let's get some author details from the author ID
		$author_id = $post->post_author;
		$author_data = get_userdata( $author_id );
		$author_nicename = $author_data->user_nicename;

		// Fetch which event category they have selected
		$event_categories = wp_get_post_terms( $post_id, 'espresso_event_categories', array( 'fields' => 'names' ) );

		if ( empty( $event_categories ) ) {
			$messages[] = 'No event category was selected.';
			$event_categories = '';
		} else {
			$event_categories = implode( ', ', $event_categories );
		}

		// Look at what Venue was selected
		if ( 0 === absint( $_POST['venue_id'] ) ) {
			$messages[] = 'No venue was selected.';
		}

		// Build the message for the email.
		$message = "Dear events team, \r\n \r\n";
		$message .= $author_nicename . ' has submitted an event for review. You can view, edit and publish this event at the following URL:' . "\r\n \r\n";
		$message .= "<a href='$post_edit_link' title='Edit event $post_title'>$post_edit_link</a>\r\n \r\n";

		if ( ! empty( $messages ) ) {
			$message .= "Here are some things to note about this event: \r\n \r\n";
			$message .= implode( '<br />', $messages );
		}

		$subject = 'A new event has been submitted for approval: ' . $post_title;
		$to = get_option( 'admin_email' );

		// Send the mail
		wp_mail( $to, $subject, $message );

	}/* transition_post_status__send_email_when_event_submitted() */


	/**
	 * Change the 'Available Tickets' title on the ticket selector
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $title - The title of the ticket selector
	 * @return (string) Modified title
	 */

	public function FHEE__ticket_selector_chart_template__table_header_available_tickets__change_available_tickets_due_to_options_whatever_that_is( $title ) {

		return 'Registration tickets';

	}/* FHEE__ticket_selector_chart_template__table_header_available_tickets__change_available_tickets_due_to_options_whatever_that_is() */


	/**
	 * We don't like 'View Details' we'd rather 'Read More'
	 *
	 * @since 1.0.0
	 *
	 * @param   -
	 * @return
	 */

	public function FHEE__EE_Ticket_Selector__display_view_details_btn__btn_text__change_view_details_button() {

		return __( 'Read More', 'ubc-ctlt-event-espresso-modifications' );

	}/* FHEE__EE_Ticket_Selector__display_view_details_btn__btn_text__change_view_details_button() */


	/**
	 * Load the chosen library when we're asking people for information for fac/dept/ dropdowns for the first time
	 *
	 * @since 1.0.0
	 *
	 * @param (int) $user_id - The ID of the user for whom we are showing this
	 * @param (object) $user_data - metadata associated with the user
	 * @return null
	 */

	public function ubc_ctlt_ee_mods_showing_need_more_info__load_chosen( $user_id, $user_data ) {

		// Enqueue chosen css and js
		wp_register_style( 'chosen', plugins_url( 'assets/css/chosen.min.css', __FILE__ ) );
		wp_enqueue_style( 'chosen' );
		wp_register_script( 'chosen', plugins_url( 'assets/js/chosen.jquery.min.js', __FILE__ ), array( 'jquery' ), null, true );
		wp_enqueue_script( 'chosen' );

		// Now register and enqueue our JS, with some extra data should we need it
		wp_register_script( 'ubc-ctlt-event-espresso-modifications', plugins_url( 'assets/js/ubc-ctlt-event-espresso-modifications.js', __FILE__ ), array( 'jquery', 'chosen' ), null, true );

		$js_vars = array();
		$js_vars['user_id'] = $user_id;

		wp_localize_script( 'ubc-ctlt-event-espresso-modifications', 'ubc_ee_js_vars', $js_vars );

		wp_enqueue_script( 'ubc-ctlt-event-espresso-modifications' );

	}/* ubc_ctlt_ee_mods_showing_need_more_info__load_chosen() */


	/**
	 * Apparently we need to change Howdy. This is probably because the events team don't really
	 * know WordPress
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $translated_text - The text we are translating
	 * @param (string) $text - The untranslated text
	 * @param (string) $domain - The text domain
	 * @return (string) Our replaced text
	 */

	function gettext__howay_howdy( $translated_text, $text, $domain ) {

		$new_message = str_replace( 'Howdy', 'Hello', $text );

		return $new_message;

	}/* gettext__howay_howdy() */


	/**
	 * Guess what, domain mapping is causing issues. WHO WOULD HAVE THUNK IT.
	 * When there's a domain-mapped site with https, domain mapping seems to make a right
	 * mess of deceiphering our urls because of the custom content directory/ssl/etc.
	 * This attempts to fix that.
	 * UPDATE: Adding the deprecated define( 'PLUGINDIR', 'content/plugins' ); to wp-config
	 * fixes most stuff, however, it doesn't work for MU plugins. This fixes that.
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $full_url - The complete URL to the plugins directory including scheme and path.
	 * @param (string) $path - Path relative to the URL to the plugins directory. Blank string if no path is specified.
	 * @param (string) $plugin - The plugin file path to be relative to. Blank string if no plugin is specified.
	 * @return (string) the full url, unmodified (as we're just testing if this is a plugin we need to unmung later)
	 */

	public function plugins_url__fix_domain_mapped_urls( $full_url, $path = '', $plugin = '' ) {

		// Only do this if domain mapping is active
		if ( ! defined( 'DOMAIN_MAPPING' ) || 1 !== DOMAIN_MAPPING ) {
			return $full_url;
		}
		// Test if the $full_url contains a reference to mu-plugins
		if ( false === stripos( $full_url, 'content/mu-plugins' ) ) {
			return $full_url;
		}
		static::$original_plugin_urls_before_domain_mapping = array(
			'full_url' => $full_url,
			'path' => $path,
			'plugin' => $plugin,
		);

		return $full_url;

	}/* plugins_url__fix_domain_mapped_urls() */


	/**
	 * In plugins_url__fix_domain_mapped_urls() we set a class property which is the ORIGINAL
	 * stuff passed through from WordPress before domain mapping has a chance to make a mess of it.
	 * In this method, we check that we're in a plugin that needs fixing (i.e. a mu-plugin) and then
	 * cancel out the munging that DM does by returning the original url stored in the class property
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $full_url - The complete URL to the plugins directory including scheme and path.
	 * @param (string) $path - Path relative to the URL to the plugins directory. Blank string if no path is specified.
	 * @param (string) $plugin - The plugin file path to be relative to. Blank string if no plugin is specified.
	 * @return (string) the full url, potentially modified if we need to unmung domain mapping
	 */

	public function plugins_url__fix_domain_mapped_urls_after_dm_munging( $full_url, $path, $plugin ) {

		// If this is an empty array, it's not a plugin that's being 'fixed'
		if ( empty( static::$original_plugin_urls_before_domain_mapping ) ) {
			return $full_url;
		}

		// WordPress gets it right, originally. Domain mapping mungs it. So just unmung it.
		$original = static::$original_plugin_urls_before_domain_mapping;
		static::$original_plugin_urls_before_domain_mapping = array();

		return $original['full_url'];

	}/* plugins_url__fix_domain_mapped_urls_after_dm_munging() */


	/**
	 * 4.9.42.p changed the way custom questions are sent through and this meant no
	 * answers were saved, meaning no one could register. That's just super. Fortunately,
	 * they have a filter, which means we can do the heavy lifting ourselves, to work out
	 * the correct ID from the question, and then save the value.
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function FHEE__EE_SPCO_Reg_Step_Attendee_Information___save_registration_form_input__save_reg_qs( $process, $registration, $form_input, $input_value, $this_object ) {

		// something like 'fname' or 'fac' or 'attas'
		$answer_cache_id = $this_object->checkout->reg_url_link ? $form_input . '-' . $registration->reg_url_link() : $form_input;

		global $wpdb;
		$question_table 		= $wpdb->prefix . 'esp_question';

		foreach ( $registration->answers() as $answer ) {

			// Note: Sometimes the qst_system is blank in the db. If so it needs to be made
			// to match the QST_admin_label in order for this to function correctly.
			$question_syt = $wpdb->get_var(
				$wpdb->prepare(
					"SELECT qst_system FROM $question_table WHERE `QST_ID` = %d",
					$answer->question_ID()
				)
			);

			if ( $question_syt === $answer_cache_id ) {
				$answer->set_value( $input_value );
				$result = $answer->save();
			}
		}

		return true;

	}/* FHEE__EE_SPCO_Reg_Step_Attendee_Information___save_registration_form_input__save_reg_qs() */

    public function AHEE__EE_Registration__set_status__after_update__fix_attendee_meta ( $reg_object ) {

        // Use the Registration ID to get the answers to the custom questions
        // i.e. QST_ID 1 = fname, QST_ID 2 = lname etc.
	    $reg_ID = $reg_object->ID();

	    global $wpdb;
	    $answer_table 		= $wpdb->prefix . 'esp_answer';
	    $attendee_meta_table= $wpdb->prefix . 'esp_attendee_meta';

	    $answers = $wpdb->get_results(
		    $wpdb->prepare(
			    "SELECT * FROM $answer_table WHERE `REG_ID` = %d",
			    $reg_ID
		    )
	    );

	    $first_name = false;
	    $last_name = false;
	    $email = false;

	    foreach ( $answers as $answer ) {

	        switch ( $answer->QST_ID ) {
		        case 1:
			        $first_name = sanitize_text_field( $answer->ANS_value );
			        break;
		        case 2:
			        $last_name = sanitize_text_field( $answer->ANS_value );
			        break;
		        case 3:
			        $email = sanitize_email( $answer->ANS_value );
			        break;

                default:
			        break;
            }
        }

	    $attendee_object = $reg_object->attendee();
	    $attendee_ID = $attendee_object->ID();

        $wpdb->query(
	        $wpdb->prepare(
		        "UPDATE $attendee_meta_table SET ATT_fname = %s WHERE ATT_ID = %d",
		        $first_name,
		        $attendee_ID
	        )
        );

	    $wpdb->query(
		    $wpdb->prepare(
			    "UPDATE $attendee_meta_table SET ATT_lname = %s WHERE ATT_ID = %d",
			    $last_name,
			    $attendee_ID
		    )
	    );

	    $wpdb->query(
		    $wpdb->prepare(
			    "UPDATE $attendee_meta_table SET ATT_email = %s WHERE ATT_ID = %d",
			    $email,
			    $attendee_ID
		    )
	    );

    }/* AHEE__EE_Registration__set_status__after_update__fix_attendee_meta() */


	/**
	 * The 'to' field in the event registration emails isn't being set correctly. Which is
	 * strange.
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function wp_mail__fix_to_field_for_event_reg_emails( $atts ) {

		// @todo: This needs to be dynamic

		$is_okanagan = false;

		if ( isset( $_SERVER ) && isset( $_SERVER['SERVER_NAME'] ) && ( 'ctlevents-verf.ok.ubc.ca' === $_SERVER['SERVER_NAME'] || 'ctlevents.ok.ubc.ca' === $_SERVER['SERVER_NAME'] ) ) {
			// We're in the Okanagan.
			$is_okanagan = true;
		}

		// Email to registrant
		if (
		        'UBC-CTLT: Event Registration Confirmation' === $atts['subject'] ||
                'Faculty ISW Registration - Pending Approval' === $atts['subject'] ||
                'Course Design Intensive - Pending Approval' === $atts['subject'] ||
                'UBC-CTLT: Grad ISW Waitlist Registration Confirmation' === $atts['subject'] ||
                'UBC-CTLT: Facilitation Workshop Series Registration & Pre-Survey' === $atts['subject'] ||
				'Your Event Registration Details' === $atts['subject'] ||
                'UBC-CTLT: Exploring Indigenous Perspectives in Teaching Practice and Learning Design Series Registration' === $atts['subject']
        ) {
			if ( empty( $atts['to'] ) ) {
				$current_user = wp_get_current_user();
				$atts['to'] = array( $current_user->data->user_email );
			}
		}

		// Email to admin
		if (
		        'Event Registration Details' === $atts['subject'] ||
                'Faculty ISW Registration' === $atts['subject'] ||
                'Course Design Intensive Registration' === $atts['subject']
        ) {
			if ( empty( $atts['to'] ) ) {
				if ( $is_okanagan ) {
					$atts['to'] = array( 'ctl.ubco@ubc.ca' );
				} else {
					$atts['to'] = array( 'ctlt.events@ubc.ca' );
				}
			}
        }

		return $atts;

	}/* wp_mail__fix_to_field_for_event_reg_emails() */

}/* class UBC_CTLT_EE_Mods */

// Initialize ourselves
add_action( 'plugins_loaded', 'ubc_ctlt_event_espresso_modifications_initialize' );

/**
 * On plugins_loaded initialize our class to add modifcations to EE Core
 *
 * @since 1.0.0
 *
 * @param null
 * @return null
 */

function ubc_ctlt_event_espresso_modifications_initialize() {

	$UBC_CTLT_EE_Mods = new UBC_CTLT_EE_Mods;
	$UBC_CTLT_EE_Mods->init();

}/* ubc_ctlt_event_espresso_modifications_initialize() */
